@@ -7,5 +7,3 @@
     :copyright: (c) 2010 by Sharoon Thomas.
     :license: AGPL, see LICENSE for more details
 '''
-from .shipping_package import ShipmentConfirm, ShipmentAccept, ShipmentVoid
-from .shipping_package import PyUPSException
